// Copyright Earl Warren <contact@earl-warren.org>
// Copyright Loïc Dachary <loic@dachary.org>
// SPDX-License-Identifier: MIT

package cmd

import (
	"context"

	"code.forgejo.org/f3/gof3/v3/logger"

	"github.com/urfave/cli/v2"
)

var Version = "development"

func SetVerbosity(ctx context.Context, verbosity int) {
	l := logger.ContextGetLogger(ctx)
	switch verbosity {
	case 0:
		l.SetLevel(logger.Info)
	default:
		l.SetLevel(logger.Trace)
	}
}

func NewApp(ctx context.Context) *cli.App {
	app := cli.NewApp()
	app.Name = "F3"
	app.Usage = "Friendly Forge Format"
	app.Description = `Friendly Forge Format`
	app.Version = Version
	app.Commands = []*cli.Command{
		CreateCmdMirror(ctx),
	}

	defaultFlags := []cli.Flag{
		&cli.BoolFlag{
			Name:  "verbose",
			Usage: "increase the verbosity level",
		},
		cli.VersionFlag,
	}
	app.Before = func(c *cli.Context) error {
		SetVerbosity(ctx, c.Count("verbose"))
		return nil
	}

	app.Flags = append(app.Flags, defaultFlags...)
	app.EnableBashCompletion = true
	return app
}
