// Copyright Earl Warren <contact@earl-warren.org>
// Copyright Loïc Dachary <loic@dachary.org>
// SPDX-License-Identifier: MIT

package forgejo

import (
	"context"
	"fmt"

	"code.forgejo.org/f3/gof3/v3/f3"
	helpers_repository "code.forgejo.org/f3/gof3/v3/forges/helpers/repository"
	"code.forgejo.org/f3/gof3/v3/id"
	f3_tree "code.forgejo.org/f3/gof3/v3/tree/f3"
	"code.forgejo.org/f3/gof3/v3/tree/generic"
)

type repository struct {
	common

	name string
	h    helpers_repository.Interface
	f    *f3.Repository
}

func (o *repository) SetNative(repository any) {
	o.name = repository.(string)
}

func (o *repository) GetNativeID() string {
	return o.name
}

func (o *repository) NewFormat() f3.Interface {
	return &f3.Repository{}
}

func (o *repository) ToFormat() f3.Interface {
	return &f3.Repository{
		Common:    f3.NewCommon(o.GetNativeID()),
		Name:      o.GetNativeID(),
		FetchFunc: o.f.FetchFunc,
	}
}

func (o *repository) FromFormat(content f3.Interface) {
	f := content.Clone().(*f3.Repository)
	o.f = f
	o.f.SetID(f.Name)
	o.name = f.Name
}

func (o *repository) Get(ctx context.Context) bool {
	return o.h.Get(ctx)
}

func (o *repository) Put(ctx context.Context) id.NodeID {
	return o.upsert(ctx)
}

func (o *repository) Patch(ctx context.Context) {
	o.upsert(ctx)
}

func (o *repository) upsert(ctx context.Context) id.NodeID {
	o.Trace("%s", o.GetNativeID())
	o.h.Upsert(ctx, o.f)
	return id.NewNodeID(o.f.Name)
}

func (o *repository) urlToRepositoryURL(url string) string {
	owner := f3_tree.GetOwnerName(o.GetNode())
	project := f3_tree.GetProjectName(o.GetNode())
	repositoryURL := fmt.Sprintf("%s/%s/%s", url, owner, project)
	if o.f.GetID() == f3.RepositoryNameWiki {
		repositoryURL += ".wiki"
	}
	return repositoryURL
}

func (o *repository) SetFetchFunc(fetchFunc func(ctx context.Context, destination string, internalRefs []string)) {
	o.f.FetchFunc = fetchFunc
}

func (o *repository) GetRepositoryURL() string {
	return o.urlToRepositoryURL(o.getURL())
}

func (o *repository) GetRepositoryPushURL() string {
	return o.urlToRepositoryURL(o.getPushURL())
}

func (o *repository) GetRepositoryInternalRefs() []string {
	return []string{"refs/pull/*"}
}

func newRepository(ctx context.Context) generic.NodeDriverInterface {
	r := &repository{
		f: &f3.Repository{},
	}
	r.h = helpers_repository.NewHelper(r)
	return r
}
